@@ -1,3 +1,4 @@
+// Copyright 2025, 2026 Element Creations Ltd.
 // Copyright 2024, 2025 New Vector Ltd.
 //
 // SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-Element-Commercial
@@ -43,11 +44,6 @@ pub struct JobContext {
     pub queue_name: String,
     pub attempt: usize,
     pub start: Instant,
-
-    #[expect(
-        dead_code,
-        reason = "we're not yet using this, but will be in the future"
-    )]
     pub cancellation_token: CancellationToken,
 }
 
@@ -124,8 +120,15 @@ where
 }
 
 #[async_trait]
-pub trait RunnableJob: FromJob + Send + 'static {
+pub trait RunnableJob: Send + 'static {
     async fn run(&self, state: &State, context: JobContext) -> Result<(), JobError>;
+
+    /// Allows the job to set a timeout for its execution. Jobs should then look
+    /// at the cancellation token passed in the [`JobContext`] to handle
+    /// graceful shutdowns.
+    fn timeout(&self) -> Option<std::time::Duration> {
+        None
+    }
 }
 
 fn box_runnable_job<T: RunnableJob + 'static>(job: T) -> Box<dyn RunnableJob> {
@@ -187,6 +190,22 @@ fn retry_delay(attempt: usize) -> Duration {
 type JobResult = (std::time::Duration, Result<(), JobError>);
 type JobFactory = Arc<dyn Fn(JobPayload) -> Box<dyn RunnableJob> + Send + Sync>;
 
+/// This is a fake job we use to consume jobs from deprecated queues
+struct DeprecatedJob;
+
+#[async_trait]
+impl RunnableJob for DeprecatedJob {
+    async fn run(&self, _state: &State, context: JobContext) -> Result<(), JobError> {
+        tracing::warn!(
+            job.id = %context.id,
+            job.queue.name = context.queue_name,
+            "Consumed a job from a deprecated queue, which can happen after version upgrades. This did nothing other than removing the job from the queue."
+        );
+
+        Ok(())
+    }
+}
+
 struct ScheduleDefinition {
     schedule_name: &'static str,
     expression: Schedule,
@@ -290,7 +309,9 @@ impl QueueWorker {
         })
     }
 
-    pub(crate) fn register_handler<T: RunnableJob + InsertableJob>(&mut self) -> &mut Self {
+    pub(crate) fn register_handler<T: RunnableJob + InsertableJob + FromJob>(
+        &mut self,
+    ) -> &mut Self {
         // There is a potential panic here, which is fine as it's going to be caught
         // within the job task
         let factory = |payload: JobPayload| {
@@ -303,6 +324,13 @@ impl QueueWorker {
         self
     }
 
+    /// Register a queue name as deprecated, which will consume leftover jobs
+    pub(crate) fn register_deprecated_queue(&mut self, queue_name: &'static str) -> &mut Self {
+        let factory = |_payload: JobPayload| box_runnable_job(DeprecatedJob);
+        self.tracker.factories.insert(queue_name, Arc::new(factory));
+        self
+    }
+
     pub(crate) fn add_schedule<T: InsertableJob>(
         &mut self,
         schedule_name: &'static str,
@@ -722,6 +750,9 @@ impl QueueWorker {
     ///
     /// This function can fail if the database connection fails.
     pub async fn process_all_jobs_in_tests(&mut self) -> Result<(), QueueRunnerError> {
+        // In case we haven't setup the schedules yet
+        self.setup_schedules().await?;
+
         // I swear, I'm the leader!
         self.am_i_leader = true;
 
@@ -849,14 +880,46 @@ impl JobTracker {
                     // We should never crash, but in case we do, we do that in the task and
                     // don't crash the worker
                     let job = factory.expect("unknown job factory")(payload);
+
+                    let timeout = job.timeout();
+                    // If there is a timeout set on the job, spawn a task which will cancel the
+                    // CancellationToken once the timeout is reached
+                    if let Some(timeout) = timeout {
+                        let context = context.clone();
+
+                        // It's fine to spawn this task without tracking it, as it is quite
+                        // lightweight and has no reason to crash.
+                        tokio::spawn(
+                            context
+                                .cancellation_token
+                                .clone()
+                                // This makes sure the task gets cancelled as soon as the job
+                                // finishes
+                                .run_until_cancelled_owned(async move {
+                                    tokio::time::sleep(timeout).await;
+                                    tracing::warn!(
+                                        job.id = %context.id,
+                                        job.queue.name = %context.queue_name,
+                                        "Job reached timeout, asking for cancellation"
+                                    );
+                                    context.cancellation_token.cancel();
+                                }),
+                        );
+                    }
+
                     tracing::info!(
                         job.id = %context.id,
                         job.queue.name = %context.queue_name,
                         job.attempt = %context.attempt,
+                        job.timeout = timeout.map(tracing::field::debug),
                         "Running job"
                     );
                     let result = job.run(&state, context.clone()).await;
 
+                    // Cancel the cancellation token to stop any timeout task
+                    // that may be running
+                    context.cancellation_token.cancel();
+
                     let Some(context_stats) =
                         LogContext::maybe_with(mas_context::LogContext::stats)
                     else {
